"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mountCallback = exports.mount = void 0;
/// <reference types="cypress" />
var test_utils_1 = require("@vue/test-utils");
var stripIndent = require('common-tags').stripIndent;
// mountVue options
var defaultOptions = [
    'vue',
    'extensions',
    'style',
    'stylesheets',
];
function checkMountModeEnabled() {
    // @ts-ignore
    if (Cypress.spec.specType !== 'component') {
        throw new Error("In order to use mount or unmount functions please place the spec in component folder");
    }
}
var registerGlobalComponents = function (Vue, options) {
    var globalComponents = Cypress._.get(options, 'extensions.components');
    if (Cypress._.isPlainObject(globalComponents)) {
        Cypress._.forEach(globalComponents, function (component, id) {
            Vue.component(id, component);
        });
    }
};
var installFilters = function (Vue, options) {
    var filters = Cypress._.get(options, 'extensions.filters');
    if (Cypress._.isPlainObject(filters)) {
        Object.keys(filters).forEach(function (name) {
            Vue.filter(name, filters[name]);
        });
    }
};
var installPlugins = function (Vue, options, props) {
    var plugins = Cypress._.get(props, 'plugins') ||
        Cypress._.get(options, 'extensions.use') ||
        Cypress._.get(options, 'extensions.plugins') ||
        [];
    // @ts-ignore
    plugins.forEach(function (p) {
        Array.isArray(p) ? Vue.use.apply(Vue, p) : Vue.use(p);
    });
};
var installMixins = function (Vue, options) {
    var mixins = Cypress._.get(options, 'extensions.mixin') ||
        Cypress._.get(options, 'extensions.mixins');
    if (Cypress._.isArray(mixins)) {
        mixins.forEach(function (mixin) {
            Vue.mixin(mixin);
        });
    }
};
// @ts-ignore
var hasStore = function (_a) {
    var store = _a.store;
    return store && store._vm;
};
var forEachValue = function (obj, fn) {
    return Object.keys(obj).forEach(function (key) { return fn(obj[key], key); });
};
var resetStoreVM = function (Vue, _a) {
    var store = _a.store;
    // bind store public getters
    store.getters = {};
    var wrappedGetters = store._wrappedGetters;
    var computed = {};
    forEachValue(wrappedGetters, function (fn, key) {
        // use computed to leverage its lazy-caching mechanism
        computed[key] = function () { return fn(store); };
        Object.defineProperty(store.getters, key, {
            get: function () { return store._vm[key]; },
            enumerable: true,
        });
    });
    store._watcherVM = new Vue();
    store._vm = new Vue({
        data: {
            $$state: store._vm._data.$$state,
        },
        computed: computed,
    });
    return store;
};
/**
 * Direct Vue errors to the top error handler
 * where they will fail Cypress test
 * @see https://vuejs.org/v2/api/#errorHandler
 * @see https://github.com/cypress-io/cypress/issues/7910
 */
function failTestOnVueError(err, vm, info) {
    console.error("Vue error");
    console.error(err);
    console.error('component:', vm);
    console.error('info:', info);
    window.top.onerror(err);
}
/**
 * Mounts a Vue component inside Cypress browser.
 * @param {object} component imported from Vue file
 * @example
 *  import Greeting from './Greeting.vue'
 *  import { mount } from 'cypress-vue-unit-test'
 *  it('works', () => {
 *    // pass props, additional extensions, etc
 *    mount(Greeting, { ... })
 *    // use any Cypress command to test the component
 *    cy.get('#greeting').should('be.visible')
 *  })
 */
exports.mount = function (component, optionsOrProps) {
    if (optionsOrProps === void 0) { optionsOrProps = {}; }
    checkMountModeEnabled();
    var options = Cypress._.pick(optionsOrProps, defaultOptions);
    var props = Cypress._.omit(optionsOrProps, defaultOptions);
    // display deprecation warnings
    if (options.vue) {
        console.warn(stripIndent(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n      [DEPRECATION]: 'vue' option has been deprecated.\n      'node_modules/vue/dis/vue' is always used.\n      Please remove it from your 'mountVue' options."], ["\n      [DEPRECATION]: 'vue' option has been deprecated.\n      'node_modules/vue/dis/vue' is always used.\n      Please remove it from your 'mountVue' options."]))));
    }
    return cy
        .window({
        log: false,
    })
        .then(function (win) {
        var localVue = test_utils_1.createLocalVue();
        // @ts-ignore
        win.Vue = localVue;
        localVue.config.errorHandler = failTestOnVueError;
        // set global Vue instance:
        // 1. convenience for debugging in DevTools
        // 2. some libraries might check for this global
        // appIframe.contentWindow.Vue = localVue
        // refresh inner Vue instance of Vuex store
        // @ts-ignore
        if (hasStore(component)) {
            // @ts-ignore
            component.store = resetStoreVM(localVue, component);
        }
        // @ts-ignore
        var document = cy.state('document');
        document.body.innerHTML = '';
        var el = document.getElementById('cypress-jsdom');
        // If the target div doesn't exist, create it
        if (!el) {
            var div = document.createElement('div');
            div.id = 'cypress-jsdom';
            document.body.appendChild(div);
            el = div;
        }
        if (typeof options.stylesheets === 'string') {
            options.stylesheets = [options.stylesheets];
        }
        if (Array.isArray(options.stylesheets)) {
            options.stylesheets.forEach(function (href) {
                var link = document.createElement('link');
                link.type = 'text/css';
                link.rel = 'stylesheet';
                link.href = href;
                el.append(link);
            });
        }
        if (options.style) {
            var style = document.createElement('style');
            style.appendChild(document.createTextNode(options.style));
            el.append(style);
        }
        var componentNode = document.createElement('div');
        el.append(componentNode);
        // setup Vue instance
        installFilters(localVue, options);
        installMixins(localVue, options);
        // @ts-ignore
        installPlugins(localVue, options, props);
        registerGlobalComponents(localVue, options);
        // @ts-ignore
        props.attachTo = componentNode;
        var wrapper = localVue.extend(component);
        var VTUWrapper = test_utils_1.mount(wrapper, __assign({ localVue: localVue }, props));
        Cypress.vue = VTUWrapper.vm;
        Cypress.vueWrapper = VTUWrapper;
    });
};
/**
 * Helper function for mounting a component quickly in test hooks.
 * @example
 *  import {mountCallback} from 'cypress-vue-unit-test'
 *  beforeEach(mountVue(component, options))
 */
exports.mountCallback = function (component, options) { return function () { return exports.mount(component, options); }; };
var templateObject_1;
